/*----------------------------------------------------------
*                                      
*    __  __                      
*   | | / /__  __ _____    ___    __   _____  ___
*   |  / / \ \/ //     \  /   \ /`__'\|  __| /   \
*   |  \ \  \  // /| |\ \|  O  |  ___/| |  `|  O  | 
*   |_| \_\  |_|_/ |_| \_\\_/|_|\___\ |_|    \_/|_|
*                  
*
*   Kymaera (c) 2004 
*	Universal Arcade Cabinet Front End
*
*	Steve Hernandez: steveh@crosslink.net 
*   Windows, Linux and Dos version
*   See the Readme.txt and License.txt for more information          
*
*
* -----------------------------------------------------------
*/


#include "crc.h"
#include <stdio.h>
#include "string.h"
#include <ctype.h> 

#define	 MAGIC_32	0xDEBB20E3      // CRC-32 magic cookie

CRC g_CRC32 ;						// the most commonly used CRC function
									// we only need one !

//------------------------------------------------------------------------------------------------------------------------------
//  Constructors
CRC::CRC( void )
{
	m_polynomial = (unsigned int) CRC32;	
	InitTable( m_polynomial ) ;
}


CRC::CRC( unsigned long polynomial )
{
	m_polynomial = polynomial ;
	InitTable( m_polynomial ) ;
}


//------------------------------------------------------------------------------------------------------------------------------
//  Destructor
CRC::~CRC( void )
{
}



//------------------------------------------------------------------------------------------------------------------------------
//  Methods




//------------------------------------------------------------------------------------------------------------------------------
///	Calc()
///
///      Calculate CRC-32 for input stream.  *** BYTE-WISE ***
//------------------------------------------------------------------------------------------------------------------------------
unsigned long CRC::Calc( const void *pData, const long nLen, unsigned long crc ) const
{
	long nSize = nLen ;
	const unsigned char *pBuf = (const unsigned char *) pData;

	while ( nSize-- )
		crc = (crc >> 8) ^ m_table[ (*pBuf++ ^ crc) & 0xFF ];

	return crc;
}

//------------------------------------------------------------------------------------------------------------------------------
///	Calcstr()
///
///      Calculate CRC-32 for input string this is case senstive.  *** BYTE-WISE ***
//------------------------------------------------------------------------------------------------------------------------------
unsigned long CRC::Calcstr( const char *pData, unsigned long crc ) const
{
	while ( *pData != '\0' )
		crc = (crc >> 8) ^ m_table[ (*pData++ ^ crc) & 0xFF ];

	return crc;
}

//------------------------------------------------------------------------------------------------------------------------------
///	Calcistr()
///
///      Calculate CRC-32 for input string this is not case senstive.  *** BYTE-WISE ***
//------------------------------------------------------------------------------------------------------------------------------
unsigned long CRC::Calcistr( const char *pData, unsigned long crc ) const
{
	while ( *pData != '\0' )
		crc = (crc >> 8) ^ m_table[ (tolower(*pData++) ^ crc) & 0xFF ];

	return crc;
}

//------------------------------------------------------------------------------------------------------------------------------
//	CrcInit()
//
//      Initialize the CRC table.
//------------------------------------------------------------------------------------------------------------------------------
void CRC::InitTable(  unsigned long polynomial )
{
	for (int i=0; i<256; i++)
	{
		unsigned long  crc = i;

		for (int j=8; j>0; j--)
		{
			int V = crc & 1;		// save carry
			crc >>= 1;
			if (V)  crc ^= polynomial;
		}

		m_table[i] = crc;
	}

	unsigned char gen[] = { 0,0,0,0 } ;
	m_magic = Calc( gen, 4 ) ;
}


