/*----------------------------------------------------------
*                                      
*    __  __                      
*   | | / /__  __ _____    ___    __   _____  ___
*   |  / / \ \/ //     \  /   \ /`__'\|  __| /   \
*   |  \ \  \  // /| |\ \|  O  |  ___/| |  `|  O  | 
*   |_| \_\  |_|_/ |_| \_\\_/|_|\___\ |_|    \_/|_|
*                  
*
*   Kymaera (c) 2004 
*	Universal Arcade Cabinet Front End
*
*	Steve Hernandez: steveh@crosslink.net 
*   Windows, Linux and Dos version
*   See the Readme.txt and License.txt for more information          
*
*
* -----------------------------------------------------------
*/


#ifndef CRC_H
#define CRC_H

#define CRC_INIT	((unsigned long) ~0)

enum
{
	CRC12 = 0x080D,		// CRC-12 polynomial	g(x) = X^12+X^11+X^3+X^2+X+1
	CRC14 = 0x202B,		// CRC-14 polynomial	g(x) = X^14+X^13+X^5+X^3+X^2+1 = (X+1)(20033F)
	CRC16 = 0x8005,		// CRC-16 polynomial	g(x) = X^16+X^15+X^2+1
	CCITT = 0x1021,		// CRC-16 polynomial	g(x) = X^16+X^12+X^5+1
	CRC32 = 0xEDB88320,	// CRC-32 polynomial	g(x) = X^32+X^26+X^23+X^22+X^16+X^12+X^11+X^10+X^8+X^7+X^5+X^4+X^2+X+1
} ;

class CRC
{

public:
	CRC() ;
	CRC( const unsigned long polynomial ) ;

	~CRC() ;

    // Member functions
	unsigned long Calc		( const void *pBuf, const long nLen, unsigned long crc = CRC_INIT ) const ;
	unsigned long Calcstr	( const char *pString, unsigned long crc = CRC_INIT ) const;
	unsigned long Calcistr	( const char *pString, unsigned long crc = CRC_INIT ) const;

protected:

private:
	// Private copy constructor and assignment operator undefined.
	CRC( const  CRC& obj );
	CRC& operator=( const  CRC & obj );

	void InitTable( unsigned long polynomial ) ;
	
    // Data members
	unsigned long m_table[ 256 ] ;
	unsigned long m_polynomial ;
	unsigned long m_magic ;



};

extern CRC g_CRC32 ;

#endif  //CRC_H

