/*----------------------------------------------------------
*                                      
*    __  __                      
*   | | / /__  __ _____    ___    __   _____  ___
*   |  / / \ \/ //     \  /   \ /`__'\|  __| /   \
*   |  \ \  \  // /| |\ \|  O  |  ___/| |  `|  O  | 
*   |_| \_\  |_|_/ |_| \_\\_/|_|\___\ |_|    \_/|_|
*                  
*
*   Kymaera (c) 2004 
*	Universal Arcade Cabinet Front End
*
*	Steve Hernandez: steveh@crosslink.net 
*   Windows, Linux and Dos version
*   See the Readme.txt and License.txt for more information          
*
*
* -----------------------------------------------------------
*/

#include "allegro.h"
#ifdef WIN32
#include <winalleg.h>
#include "Shlwapi.h"
#endif

#include "fileutil.h"
#include "string.h"
#include "stdio.h"


#include "dirmanager.h"

#ifdef WIN32
	#include "io.h"
	#include "direct.h"
#endif	

#include "time.h"
#include <string>

#ifdef LINUX
	#include <sys/stat.h> 
	#include <unistd.h>  
	//for file time
#endif

#ifdef DJGPP
	#include "dir.h"
	#include <sys/stat.h>

#endif

using namespace std;
void ToUpper(std::string *str1){
	std::string str;
	str = *str1;
	for ( unsigned int g = 0 ; g < str.size() ; g++ ){
		if ( str[g] >= 'a' && str[g] <= 'z' ){
			str[g] -= 'a' - 'A';
		}
	}
	*str1 = str;
}

bool HasExtension(char *filename){
	if(filename[strlen(filename)-4] == '.'){
		return true;
	}
	return false;
}

void RemoveExtension(char *filein, char *fileout){
    if(HasExtension(filein)){
    	strncpy(fileout,filein,strlen(filein)-4);
		fileout[strlen(filein)-4]=0;
    }else{
        strcpy(fileout,filein);
    }
}


bool FileExists(char *f){
	if(!f)return false;
	return DirMgr->FileExists(f);
}

void GetExtension(const char *srcfile,char *outext){
	if(HasExtension((char *)srcfile)){
		strcpy(outext,srcfile + strlen(srcfile) - 3);
	}else{
		strcpy(outext,"");
	}
}

void ChangeExtension(char *in,char *newext,char *out){ // change the extension of the file
	strcpy(out,in);
	strcpy(out + strlen(out) - 3,newext);
}


long GetFileLength(char *name )
{
#ifdef WIN32
	long eof_ftell;
    FILE *file;
    file = fopen( name, "r" );
    if ( file == NULL )
        return( 0L );
    fseek( file, 0L, SEEK_END );
    eof_ftell = ftell( file );
    fclose( file );
    return( eof_ftell );
#else
	struct stat s_buf;
	stat(name, &s_buf); 	
	return s_buf.st_size;
#endif
}

long GetFileLength(FILE *file )
{
    long eof_ftell;
    if ( file == NULL )
        return( 0L );
    fseek( file, 0L, SEEK_END );
    eof_ftell = ftell( file );
	fseek( file, 0L, SEEK_SET ); // rewind back to beginning
    return( eof_ftell );
}
string FileDateTime(string filename){
#ifdef WIN32 
	intptr_t handle;
	_finddata_t fd;
	handle = _findfirst(filename.c_str(),&fd);		
	_findclose(handle);
	return string (ctime(&fd.time_write));
#else	// DJGPP and LINUX
	struct stat s_buf;
	stat(filename.c_str(), &s_buf); 
	return string(ctime(&s_buf.st_mtime));
#endif
}



void FileCopy(char *inf, char *outf){
	FILE *infi;
	FILE *outfi;
	int dat;
	infi = fopen(inf,"rb");
	if(!infi)return;
	outfi = fopen(outf,"wb");
	if(!outfi){
		fclose(infi);
		return;
	}
	bool done = false;
	while (!done){
		dat = getc(infi);
		putc(dat,outfi);
		if(dat==EOF)done = true;
	}
	fclose(infi);
	fclose(outfi);
}

// this returns the path to the file
void GetFilePath(char *filen,char *retpath){
	char t;
	int len =0;
	int done =0;
	strcpy(retpath,filen);
	len =(int) strlen(filen);
	if(len==0)return;
	len -=1;
	while(!done){
		t = filen[len];		   
#ifndef LINUX		
		if(t == '\\'){
#else 
        if(t == '/'){
#endif		
			done =1;
			break;
		}
		len -= 1;
	}
	retpath[len + 1] = '\0';
}
void FixPath(char *pathname){
#ifndef LINUX
	if(pathname[strlen(pathname)-1] != '\\'){		
		strcat(pathname,"\\");//add backslash
	}
#else
	if(pathname[strlen(pathname)-1] != '/'){		
		strcat(pathname,"/");//add forwardslash
	}
#endif
}
void GetFileTitle(char *filen,char *retname){
	//GetFileName	(filen,t1); // first get rid of the path
	strncpy(retname,filen,strlen(filen)-4);
	retname[strlen(filen)-4] = '\0';
}
//this returns the filename with no path
void GetFileName(char *filen,char *retname){
	char t;
	int len =0,lens=0;
	int done =0;
	//strcpy(retpath,filen);
	len = (int)strlen(filen);	
	len -=1;
	lens +=1;
	while(!done){
		t = filen[len];
#ifndef LINUX		
		if(t == '\\'){
#else 
        if(t == '/'){
#endif	
			done =1;
			break;
		}
		len -= 1;
		lens += 1;
	}
	memcpy(&retname[0],&filen[len+1],lens-1);//copy back the name
	retname[lens-1] = '\0'; //close off string
}
// this returns the filename with no extension
void GetFileBase(char *infile,char *retfile){
	unsigned int count = 0;
	while(count < strlen(infile)){
		if(infile[count]=='.')break; // look for period
		retfile[count] = infile[count];
		count++;
	}
	retfile[count ] = '\0';
}

int MyStrLen(char *s){
	int c=0;
	if(s!=0)
	while (*s != 0){
		s++;
		c++;
	}
	return c;
}

bool MatchString(char *s1,char *s2){ // case insensitive(lcase)
	int s1len = MyStrLen(s1);
	int s2len = MyStrLen(s2);
	if(s1len!=s2len)return false;
	char s,t;
	//s = s1;
	while (*s1!=0){
		s = *s1;
		t = *s2;
		if((s >= 65)&&(s<=90)) s+=32;
		if((t >= 65)&&(t<=90)) t+=32;
		if(s!=t)return false;
		s1++;
		s2++;
	}
	return true;
}

void LoadString(string *str,FILE *fp){
	char temp[1024];
	//char *cptr = temp;
	int idx=0;
	char lastchar;
	fread(&lastchar,1,1,fp);
	while(lastchar !=0 ){
		temp[idx++] = lastchar;
		fread(&lastchar,1,1,fp);
	}
	temp[idx++] = 0;
	*str = temp;
}

void SaveString (string str,FILE *fp){
	char temp[256];
	strcpy(temp,str.c_str());
	char zero =0;
	//fwrite(temp,len,1,fp);
	fwrite(temp,strlen(temp),1,fp);
	fwrite(&zero,1,1,fp);
}

void SaveStringList(List *lst,FILE *fp){
	int num = lst->numelements;
	fwrite(&num,sizeof(int),1,fp);
	Listitem *li;
	string *str;
	for(li=lst->list;li!=0;li=li->next ){
		str = (string *)li->data;
		SaveString(*str,fp);
	}
}

void LoadStringList(List *lst,FILE *fp){
	int num;
	fread(&num,sizeof(int),1,fp);
	string *str;
	for(int c=0;c<num;c++){
		str = new string();
		LoadString(str,fp);
		lst->Add(str);
	}
}

string g_basepth;
void SetBasePath(string basepath){
	g_basepth = basepath;//..MakeLower();
	ToUpper(&g_basepth);
}
/*
string RelativePath(string pathin){
	char outpath[MAX_PATH];
	char temp1[256];
	char temp2[256];
	if(!PathRelativePathTo(outpath,g_basepth,FILE_ATTRIBUTE_DIRECTORY,pathin,FILE_ATTRIBUTE_DIRECTORY)){
		return pathin; // can't do it		
	}
	strcpy(temp1,outpath);
	if(!strnicmp(temp1,".\\",2)){ // has  
		strcpy(temp2,temp1+2);
		return string(temp2);
	}
	return string (outpath);
}
*/
/*
string RelativeFile(string pathin){
	char outpath[MAX_PATH];
	char temp1[256];
	char temp2[256];

	if(!PathRelativePathTo(outpath,g_basepth,FILE_ATTRIBUTE_DIRECTORY,pathin,FILE_ATTRIBUTE_NORMAL)){
		return pathin; // can't do it		
	}
	strcpy(temp1,outpath);
	if(!strnicmp(temp1,"\\",1)){ // has  
		strcpy(temp2,temp1+1);
		return string(temp2);
	}
	return string (outpath);
}
*/
#ifdef LINUX
#define strnicmp strncmp
#endif
string RelativeFile(string pathin){
	char tmp[255];
	char tmp2[255];

	sprintf(tmp,"%s",pathin.c_str());
	if(!strnicmp(tmp,g_basepth.c_str(),strlen(g_basepth.c_str()))){
		strcpy(tmp2,tmp + strlen(g_basepth.c_str()));
		return string(tmp2);
	}else{
		return pathin; // can't do it, leave whole
	}
}

#ifndef LINUX
void GetDirListing(string dir,List *dirlist){
	string parms;
	parms = "dir /b";
	parms += dir + g_FE->m_AppDir + "dirlist.txt";
	FILE *fp;
	system(parms.c_str());
	char tmp[255];
	string *newstring,t;
	t = g_FE->m_AppDir + "dirlist.txt";
	fp = fopen(t.c_str(),"rt");
	while(fgets(tmp,255,fp)){
		t = tmp;
		newstring = new string (t);
		dirlist->Add(newstring);
	}
	fclose(fp);
}
#endif

