/*----------------------------------------------------------
*                                      
*    __  __                      
*   | | / /__  __ _____    ___    __   _____  ___
*   |  / / \ \/ //     \  /   \ /`__'\|  __| /   \
*   |  \ \  \  // /| |\ \|  O  |  ___/| |  `|  O  | 
*   |_| \_\  |_|_/ |_| \_\\_/|_|\___\ |_|    \_/|_|
*                  
*
*   Kymaera (c) 2004 
*	Universal Arcade Cabinet Front End
*
*	Steve Hernandez: steveh@crosslink.net 
*   Windows, Linux and Dos version
*   See the Readme.txt and License.txt for more information          
*
*
* -----------------------------------------------------------
*/

#define DX3IFACEGUID
#include "kybitmap.h"
#undef DX3IFACEGUID


#include "string.h"
#include "texturemgr.h"
//#include "fblend.h"

#include "log.h"
//#include "fileutil.h"

void BestFitRect(KYRect dest,int width,int height,KYRect *out){
	float w,h,aspect;
	float outw,outh;
	int destw,desth;
	float hrw,hrh;
	w = (float)width;
	h = (float)height;
	if (width == 0) return;
	if (height == 0) return;
	aspect = w/h;
	destw = dest.Width();
	desth = dest.Height();

	outw = (float)destw; // set to the full width 
	outh = outw / aspect; // calculate the height
	if(outh > (float)desth+1){// if it is too big, fit to the height
		outh = (float)desth;
		outw = outh*aspect;//?
	}

	out->top = dest.top;
	out->left = dest.left;
	out->right = dest.left+((long)outw);
	out->bottom = dest.top+((long)outh);

	hrh = (desth - out->Height()) *.5f;
	hrw = (destw - out->Width()) *.5f;

	out->top += (long)hrh;
	out->left += (long)hrw;
	out->right += (long)hrw;
	out->bottom += (long)hrh;
}



bool KYBitmap::Load(char *filename){
	ismovie = false;

			m_pSurf = TheScreen->KYLoadMedia(filename);// load_bitmap(filename,0);
			if(m_pSurf){
				strcpy(m_filename,filename);
				isloaded = true;
				ismovie = false;
				return true;
			}


	return false;
}

void KYBitmap::Release(){



	if(m_pSurf){
		TxtMgr->RemoveRef(m_pSurf);
		m_pSurf = 0;
	}

	isloaded = false;
}
void KYBitmap::Pause(long pausetime){
	Release();
}
void KYBitmap::Restore(long restoretime){
	Load(m_filename);
}
void KYBitmap::StretchBestFit(KYRect *dst,long alpha){
	if(!this->isloaded)return;
	KYRect outrc;
	BestFitRect(*dst,Width(),Height(),&outrc);
	BITMAP *tmpbitmap;

	if(alpha != 255){
		//blit m_pSurf to the bitmap stretched
		tmpbitmap = create_bitmap(outrc.Width(),outrc.Height());
		//blit to the new bitmap
		stretch_blit(m_pSurf,tmpbitmap,0,0,Width(),Height(),0,0,outrc.Width(),outrc.Height());
		//blit the new bitmap to the screen with alpha
#ifndef LINUX
    //	fblend_trans(tmpbitmap,TheScreen->backbuffer,outrc.left,outrc.top,alpha);
#endif	
		destroy_bitmap(tmpbitmap);
	}else{
		//Log("blitting image");
		stretch_blit(m_pSurf,TheScreen->backbuffer,0,0,Width(),Height(),outrc.left,outrc.top,outrc.Width(),outrc.Height());
	}
}
void KYBitmap::StretchBestFitRotated(BITMAP *bm,KYRect *dst,float angle){
	if(!this->isloaded)return;
	KYRect outrc;
	BestFitRect(*dst,Width(),Height(),&outrc);
	float f2 = angle *.711111f; // convert to 256
	
	unsigned int iangle = (int)f2;
	
	unsigned int iscale = 1<<16;
	iangle =  iangle << 16;
//Log("Angle %f",f2);
	float fw = Width();
	float fw2 = outrc.Width();
	float fs = fw2/fw;
	fs *= 1024;
	iscale = (unsigned int )fs;
	iscale =iscale << 6;
	pivot_scaled_sprite(bm,m_pSurf,							
							outrc.left,outrc.top,
							outrc.Width()>>1,outrc.Height()>>1,
							iangle, iscale);

}
void KYBitmap::StretchBestFitTrans(KYRect *dst,long trcolor){
	if(!this->isloaded)return;
	KYRect outrc;
	BestFitRect(*dst,Width(),Height(),&outrc);

	masked_stretch_blit(m_pSurf,TheScreen->backbuffer,
                         0,0,Width(),Height(),
                         outrc.left ,outrc.top ,
						 outrc.Width(),outrc.Height());
}
