/*----------------------------------------------------------
*                                      
*    __  __                      
*   | | / /__  __ _____    ___    __   _____  ___
*   |  / / \ \/ //     \  /   \ /`__'\|  __| /   \
*   |  \ \  \  // /| |\ \|  O  |  ___/| |  `|  O  | 
*   |_| \_\  |_|_/ |_| \_\\_/|_|\___\ |_|    \_/|_|
*                  
*
*   Kymaera (c) 2004 
*	Universal Arcade Cabinet Front End
*
*	Steve Hernandez: steveh@crosslink.net 
*   Windows, Linux and Dos version
*   See the Readme.txt and License.txt for more information          
*
*
* -----------------------------------------------------------
*/

#include "allegro.h"

#include "mmdd.h"
#include "log.h"
#include "stdio.h" 
#include "crc.h"
#include "texturemgr.h"
#include <math.h>

long using_arcade=0; // set to non-zero for arcade monitor

extern "C"{
	extern int select_display_mode(int width,int height,int depth); // for arcade and tweaked modes
}
#define DEGtoRAD(x) x*0.0174532925199
double					SIN_TABLE[360];
double					COS_TABLE[360];
void InitRotationTables()
{
	int c = 0;
	double sn,cs;
	double di;
	for( double i = 0; i < 360.0; i++,c++ )
	{
		di = i * 0.0174532925199;
		sn = sinf( di );
		cs = cosf( di );
		SIN_TABLE[c] = sn;
		COS_TABLE[c] = cs;
	}
}
void Screen::Initialize(int scr_width,int scr_height,int bpp,int skn_width,int skn_height,bool windowed){
	set_color_depth(bpp);
	m_fullscreen = !windowed;
	m_bpp = bpp;
	m_width = skn_width;
	m_height = skn_height;
	int mode;

	if(windowed){	
		mode = GFX_AUTODETECT_WINDOWED;
	}else{
		mode = 	GFX_AUTODETECT_FULLSCREEN;					
	}

#ifndef WIN32
	if(using_arcade){
		if(!select_display_mode(scr_width,scr_height,bpp)){
			Log("Error selecting arcade display mode ");
		}
	}else{
		set_gfx_mode(mode,scr_width,scr_height,0,0);		
	}
#else
	set_gfx_mode(mode,scr_width,scr_height,0,0);		
#endif
		

	 backbuffer = create_bitmap(skn_width, skn_height);
	 Log("created back buffer %d %d",scr_width, scr_height);
	rcfullscreen.SetRect(0,0,scr_width,scr_height);
	rcbackbuff.SetRect(0,0,skn_width,skn_height);
	set_display_switch_mode(SWITCH_BACKGROUND);
	 if(m_bpp == 8){
		InitGlobalPallete();
	 }

}
void Screen::Pause(){
     destroy_bitmap(backbuffer);
	 set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);

}
void Screen::Resume(){
	set_color_depth(m_bpp);
#ifndef WIN32
	if(using_arcade){
		if(!select_display_mode(rcfullscreen.Width(),rcfullscreen.Height(),m_bpp)){
			Log("Error selecting arcade display mode ");
		}
	}else{
		set_gfx_mode(GFX_AUTODETECT_FULLSCREEN,rcfullscreen.Width(),rcfullscreen.Height(),0,0);		
	}
#else
	if(!m_fullscreen){
		set_gfx_mode(GFX_AUTODETECT_WINDOWED,rcfullscreen.Width(),rcfullscreen.Height(),0,0);
	}else{
		set_gfx_mode(GFX_AUTODETECT_FULLSCREEN,rcfullscreen.Width(),rcfullscreen.Height(),0,0);
	}

//	set_gfx_mode(mode,scr_width,scr_height,0,0);		
#endif
    //backbuffer = create_video_bitmap(rcbackbuff.Width(),rcbackbuff.Height());
	 backbuffer = create_bitmap(rcbackbuff.Width(),rcbackbuff.Height());
	 if(m_bpp == 8){
		InitGlobalPallete();
	 }
}


Screen::Screen(){
		m_fullscreen = false;
		framecount=0;
		backbuffer = 0;
		InitRotationTables();
}
void Screen::Cleanup(bool final){
	if(final){
		if(backbuffer){
			destroy_bitmap(backbuffer);
			backbuffer = 0;
		}
	}
}
void Screen::PageFlip(){
	stretch_blit(backbuffer,screen,0,0,backbuffer->w,backbuffer->h,0,0,rcfullscreen.Width(),rcfullscreen.Height());
	framecount++;
}

BITMAP *Screen::KYLoadMedia(char * szBitmap){
	//look for it int the texture manager first
	//if not found, load it.
	BITMAP *ret=0;
	ret = TxtMgr->Find(szBitmap);
	if(!ret){
             Log("loading image %s",szBitmap);
		ret = load_bitmap(szBitmap,0);
		//Log("loaded image");
		if(ret){
			TextureItem *ti = new TextureItem();
			ti->pSurf = ret;
			ti->m_nameCRC = g_CRC32.Calcistr(szBitmap);
			ti->m_refcount =1;
		}
		//load it and add it to the texture mamanger
	}
	return ret;
}
Screen::~Screen(){
   //   destroy_bitmap(backbuffer);
//	 set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
}

void Screen::InitGlobalPallete(){
	//create a pallete
//	PALETTE pal;
	RGB pal[256];
	Log("8bpp mode");
	generate_332_palette(pal);//set it to 332 rgb
	set_palette(pal);//set it globally
	set_color_conversion(COLORCONV_REDUCE_TO_256 | COLORCONV_KEEP_TRANS);
}
