
#ifndef MUSICSAMPLE
#define MUSICSAMPLE

#include "allegro.h"

#include "fileutil.h"

#include <string>
using namespace std;

#define DATASZ  (1<<18) /* (256k) amount of data to read from disk each time */
#define BUFSZ   (1<<19) /* (512k) size of audiostream buffer */



class MusicSample // a mostly virtual base class for music samples
{
public:
	string m_filename;
	bool	m_loaded;
	bool	m_playing;
	MusicSample(void){
		m_playing = false;
		m_loaded = false;
	}
	virtual ~MusicSample(void){};
	virtual bool Load(char *filename)=0;
	virtual void Play()=0;
	virtual void Pause()=0;
	virtual void Resume()=0;

	virtual void Stop()=0;
	virtual void Update(){};
};



class WaveSample : public MusicSample
{
public:
	SAMPLE *m_music;
	WaveSample(void){
		m_music=0;
	}
	virtual ~WaveSample(void){}
	bool Load(char *filename){
		Stop();
		m_music = load_wav(filename);
		if(m_music){
			m_filename = filename;
			m_loaded = true;
			return true;
		}else{
			return false;
		}
	}
	void Play(){
		if(m_music){
			play_sample(m_music,255,128,1000,0);
			m_playing = true;
		}
	}
	void Stop(){
		if(m_music){
			destroy_sample(m_music);
			m_music = 0;
			m_loaded = false;
			m_playing = false;
		}
	}
	void Pause(){}
	void Resume(){}
	void Update(){	}

};

class SoundFile{
public:
	MusicSample *m_pms;
	SoundFile(void){
		m_pms = 0;
	}
	~SoundFile(void){
		if(m_pms){
			delete m_pms;
			m_pms = 0;
		}
	};
	bool IsPlaying(){
		if(m_pms){
			return m_pms->m_playing;
		}else{
			return false;
		}
	}
	bool IsLoaded(){
		if(m_pms){
			return m_pms->m_loaded;
		}else{
			return false;
		}
	}
	bool Load(char *filename){
		char ext[24];
		GetExtension(filename,ext);//get the extension
		bool ret= false;		
		if(m_pms){
			delete m_pms;
			m_pms = 0;
		}
		if(!stricmp(ext,"wav")){//create the appropriate sample
			m_pms = new WaveSample();
		}
		if(m_pms)
			return m_pms->Load(filename);
		else{
			return false;
		}
	}
	void Play(){
		if(m_pms){
			m_pms->Play();
		}		
	}
	void Pause(){
		if(m_pms){
			m_pms->Pause();
		}			
	}
	void Resume(){
		if(m_pms){
			m_pms->Resume();
		}	
	}
	void Stop(){
		if(m_pms){
			m_pms->Stop();
		}		
	}
	void Update(){
		if(m_pms){
			m_pms->Update();
		}		
	}
};


#endif
